/*-------------------------------------------------------------------------------------------------\
| Solarboy Solar Bike Controller
|
| File:					BIKE.C			Mainline Code
|
| Global Routines:	Main()
|
| Local Routines:		
|
\-------------------------------------------------------------------------------------------------*/

#include	<pic.h>
#include "bike.h"

//-------------------------------------------------------------------------------------------------
//  Define Global Variables (FOR ALL ROUTINES AND MODULES)
//-------------------------------------------------------------------------------------------------

	// Note: In PICC, "char" is an byte which is by default unsigned, "unsigned int" is a 16bit quantity.
	//
	// ALL VARIABLES ARE INITIALIZED TO ZERO ON RESET (courtesy Hi-Tech PIC C startup code)
	// Nonzero initializations (marked with "*") are initialized in the function Initialize() called in main.

	bit						gbPowerUp;							// 1 = Power up mode flagged / 0 = Power up mode off
	bit						gbSystemFault;						// 1 = Horrible system error occured / 0 = Everything is fine

	bit						gbGPSOk;								// 1 = GPS operating / 0 = not operating (timer2 watchdogs by setting to zero)
	bit						gbStartTx;							// 1 = start packet transmissions / 0 = packet transmissions done
	bit						gbStartMain;						// 1 = start main's processing of packet / 0 = packet processing done
	
	bit						gbPosBattAmps;						// 1 = positive battery current (charging) / 0 = negative (discharging)
	bit						gbPosMotorAmps;					// 1 = positive motor current (running the motor) / 0 = negative (regen)

	bit						gbSpeedSensor;						// 1 = Speed sensor has gone 0 to 1 / 0 = not yet
	bit						gbCadenceSensor;					// 1 = cadence sensor has gone 0 to 1 / 0 = not yet

	unsigned char			gPacketNum;							// Which packet we're currently working on (loading)
	unsigned char			gTxPacketNum;						// Which packet we're transmitting

	signed	long			ampHours;							// amp hours in "amp ticks"
	
	unsigned char			battAmps;							// High byte of battery amps count
	unsigned char			motorAmps;							// High byte of motor amps count

	unsigned char			speed0;								// Low byte of speed time between pulses
	unsigned char			cadence0;							// Low byte of cadence time between pulses
	unsigned char			speedCadence1;						// High nibbles of speed and cadence packed together

	unsigned char			gFaultLog[16];						// An array to store the fault log in.

	unsigned bank2	char	gPacket0[PACKETLENGTH];			// Packet #0 to send out the UART
	unsigned bank2	char	gPacket1[PACKETLENGTH];			// 1
	unsigned bank3	char	gPacket2[PACKETLENGTH];			// 2
	unsigned bank3	char	gPacket3[PACKETLENGTH];			// 3

//-------------------------------------------------------------------------------------------------	
//  Define Local (to this file) Global Variables
//-------------------------------------------------------------------------------------------------

	/*
	 *  Table of CRC values for low-order byte
	 */
	const char CRCTableLo [] = {
	0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
	0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
	0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
	0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
	0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
	0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
	0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
	0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
	0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
	0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
	0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
	0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
	0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
	0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
	0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
	0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40
	};

	/*
	 *  Table of CRC values for high-order byte
	 */
	const char CRCTableHi [] = {
	0x00, 0xC0, 0xC1, 0x01, 0xC3, 0x03, 0x02, 0xC2, 0xC6, 0x06, 0x07, 0xC7, 0x05, 0xC5, 0xC4, 0x04,
	0xCC, 0x0C, 0x0D, 0xCD, 0x0F, 0xCF, 0xCE, 0x0E, 0x0A, 0xCA, 0xCB, 0x0B, 0xC9, 0x09, 0x08, 0xC8,
	0xD8, 0x18, 0x19, 0xD9, 0x1B, 0xDB, 0xDA, 0x1A, 0x1E, 0xDE, 0xDF, 0x1F, 0xDD, 0x1D, 0x1C, 0xDC,
	0x14, 0xD4, 0xD5, 0x15, 0xD7, 0x17, 0x16, 0xD6, 0xD2, 0x12, 0x13, 0xD3, 0x11, 0xD1, 0xD0, 0x10,
	0xF0, 0x30, 0x31, 0xF1, 0x33, 0xF3, 0xF2, 0x32, 0x36, 0xF6, 0xF7, 0x37, 0xF5, 0x35, 0x34, 0xF4,
	0x3C, 0xFC, 0xFD, 0x3D, 0xFF, 0x3F, 0x3E, 0xFE, 0xFA, 0x3A, 0x3B, 0xFB, 0x39, 0xF9, 0xF8, 0x38,
	0x28, 0xE8, 0xE9, 0x29, 0xEB, 0x2B, 0x2A, 0xEA, 0xEE, 0x2E, 0x2F, 0xEF, 0x2D, 0xED, 0xEC, 0x2C,
	0xE4, 0x24, 0x25, 0xE5, 0x27, 0xE7, 0xE6, 0x26, 0x22, 0xE2, 0xE3, 0x23, 0xE1, 0x21, 0x20, 0xE0,
	0xA0, 0x60, 0x61, 0xA1, 0x63, 0xA3, 0xA2, 0x62, 0x66, 0xA6, 0xA7, 0x67, 0xA5, 0x65, 0x64, 0xA4,
	0x6C, 0xAC, 0xAD, 0x6D, 0xAF, 0x6F, 0x6E, 0xAE, 0xAA, 0x6A, 0x6B, 0xAB, 0x69, 0xA9, 0xA8, 0x68,
	0x78, 0xB8, 0xB9, 0x79, 0xBB, 0x7B, 0x7A, 0xBA, 0xBE, 0x7E, 0x7F, 0xBF, 0x7D, 0xBD, 0xBC, 0x7C,
	0xB4, 0x74, 0x75, 0xB5, 0x77, 0xB7, 0xB6, 0x76, 0x72, 0xB2, 0xB3, 0x73, 0xB1, 0x71, 0x70, 0xB0,
	0x50, 0x90, 0x91, 0x51, 0x93, 0x53, 0x52, 0x92, 0x96, 0x56, 0x57, 0x97, 0x55, 0x95, 0x94, 0x54,
	0x9C, 0x5C, 0x5D, 0x9D, 0x5F, 0x9F, 0x9E, 0x5E, 0x5A, 0x9A, 0x9B, 0x5B, 0x99, 0x59, 0x58, 0x98,
	0x88, 0x48, 0x49, 0x89, 0x4B, 0x8B, 0x8A, 0x4A, 0x4E, 0x8E, 0x8F, 0x4F, 0x8D, 0x4D, 0x4C, 0x8C,
	0x44, 0x84, 0x85, 0x45, 0x87, 0x47, 0x46, 0x86, 0x82, 0x42, 0x43, 0x83, 0x41, 0x81, 0x80, 0x40
	};
	
//-------------------------------------------------------------------------------------------------	
//  Local #defines
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local Function Prototypes
//-------------------------------------------------------------------------------------------------



/*-------------------------------------------------------------------------------------------------\
| Function:		main
|
| Purpose:		Run all Mainline code
|
| Discussion:	
|
| Expects:		Interrupts to be enabled.
|					
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 10/28/01 | Borrowed from old ADG code.
\--------------------------------------------------------------------------------------------------*/

void main (void)
{

	static unsigned char	i;
	static signed long	tempLong;
	
	static signed int		tempInt;
	
	static unsigned char	tempChar1;
	static unsigned char	tempChar2;
	static unsigned char	tempChar3;
	static unsigned char	tempChar4;

	static unsigned char	faultLogPktIndex;
	
	static char 			CRCLo;
	static char 			CRCHi;
	
	
	Initialize();

	while (TRUE)
	{
		// Wait until we get the "one second" flag
		
		while(!gbStartMain)
			;

		// Ok, go nuts, it's been one second according to the GPS or TIMER2.
		// Turn off interrupts and immediate grab amphours, battery amps, motor amps.
	
		GIE = 0;

		// Get low bytes on the amp measurements; if we miss a pulse, we'll live -
		// it's a low probability that in a microsecond or two we'll get the
		// 256 rollover from TMR0 or TMR1L.
		
		tempChar1 = TMR0;
		TMR0 = 0;
		tempChar3 = TMR1L;
		TMR1L = 0;

		// If we do, however, miss a pulse then it JUST happened before we cleared
		// the two timers. However, if that count was a rollover then when we turn
		// interrupts back on, it'll add an extra count onto the MSBs of the amp
		// measurements which we don't want. So we clear the interrupt flags here.
		
		T0IF = 0;
		CCP1IF = 0;

		// Now get the high bytes
		
		tempChar2 = battAmps;
		battAmps = 0;

		tempChar4 = motorAmps;
		motorAmps = 0;

		// Before we throw out the amp counts, we need to add the extra bits in
		// TMR0 into the amp hours.
		
		if (gbPosBattAmps)
			ampHours += tempChar1;
		else
			ampHours -= tempChar1;

		tempLong = ampHours;

		// Turn interrupts back on
		
		GIE = 1;

		// Calculate and store battery amps

		if (gbPosBattAmps)
			tempInt = (tempChar2 << 8) + tempChar1;			// tempBattAmps = (tempBattAmps << 8) + tempTMR0
		else
			tempInt = -((tempChar2 << 8) + tempChar1);		// tempBattAmps = -((tempBattAmps << 8) + tempTMR0)

		Write2Packet (PKT_BATTAMP0, tempInt);
		Write2Packet (PKT_BATTAMP1, tempInt >> 8);

		// Store battery amphours
		
		Write2Packet (PKT_AMPHR0, tempLong);
		Write2Packet (PKT_AMPHR1, tempLong >> 8);
		Write2Packet (PKT_AMPHR2, tempLong >> 16);

		// Calculate and store motor amps

		if (gbPosMotorAmps)
			tempInt = (tempChar4 << 8) + tempChar3;			// tempMotorAmps = (tempMotorAmps << 8) + tempTMR1L
		else
			tempInt = -((tempChar4 << 8) + tempChar3);		// tempMotorAmps = -((tempMotorAmps << 8) + tempTMR1L)

		Write2Packet (PKT_MOTORAMP0, tempInt);
		Write2Packet (PKT_MOTORAMP1, tempInt >> 8);

		//------------------------------------------------------------------------------------
		// Combine all of the digital inputs: RC1,RC2,RC3,RC4,RC5,RB0,RB1,RB2
		// OR if gbSystemFault is set, then replace the digital byte with a revolving error count.
		// and the MSb becomes the gbSystemError.

		if (!gbSystemFault)
		{
			tempChar1 = (PORTC >> 1) & 0b00011111;		// Load in RC1,2,3,4,5
			tempChar1 |= (PORTB<<5);						// Load in RB0,1,2
		}

		else
		{
			tempChar1 = gFaultLog[faultLogPktIndex];
			tempChar1 |= 0b10000000;
			
			++faultLogPktIndex;
			if (faultLogPktIndex >= FAULTLOGLENGTH)
				faultLogPktIndex = 0;
		}

		Write2Packet (PKT_DIGITAL, tempChar1);
		
		//------------------------------------------------------------------------------------
		// Sample CH0 - 7, store in the current packet

		tempChar1 = 0;
		
		for (i = 0; i < 4 ; ++i)
		{
			ADCON0 &= 0b11000111;
			ADCON0 |= (i<<3);

			// Wait for 15us = 15us/200ns/instruction = 75 instructions 

			tempChar2 = 15;
			while (tempChar2 != 0)				// ~6 Tcyc/loop
				--tempChar2;
				
			ADGO = 1;

			while (ADGO)
				;

			Write2Packet(PKT_ANCH0+i, ADRESL);		// Write the low byte to the packet
			tempChar1 |= ADRESH<<(i<<1);				// Write the upper 2 bits to tempChar1
		}

		Write2Packet(PKT_ANCH_HIGH0, tempChar1);	// write upper bits of AN0-3

		tempChar1 = 0;
		
		for (i = 4; i < 8 ; ++i)
		{
			ADCON0 &= 0b11000111;
			ADCON0 |= (i<<3);

			// Wait for 15us = 15us/200ns/instruction = 75 instructions 

			tempChar2 = 15;
			while (tempChar2 != 0)						// ~6 Tcyc/loop
				--tempChar2;
	
			ADGO = 1;

			while (ADGO)									// Wait for ADC
				;

			tempChar3 = i - 4;
			Write2Packet((PKT_ANCH4)+tempChar3, ADRESL);		// Write the low byte to the packet
			tempChar1 |= ADRESH<<((tempChar3)<<1);				// Write the upper 2 bits to tempChar1
		}

		Write2Packet(PKT_ANCH_HIGH1, tempChar1);	// write upper bits of AN4-7


		//------------------------------------------------------------------------------------
		// Write the speed and cadence bytes and high nibble (turn off ints to prevent an
		// overwrite).
		
			GIE = 0;

			Write2Packet(PKT_SPEED, speed0);						// Write the low byte to the packet
			Write2Packet(PKT_CADENCE, cadence0);				// Write the low byte to the packet
			Write2Packet(PKT_SPDCAD_HIGH, speedCadence1);	// Write the high nibbles to the packet

			GIE = 1;

		//------------------------------------------------------------------------------------
		// Calculate the CRC-16 checksum and store.
		//
		// This code generates a 16 bit CRC using the generator polynomial of 0x18005 (yes this
		// is 17 bits, the count starts from zero). This is also known commonly as "CRC-16".
		// This algorithm tends to process the least significant bits first. Originally G.N.
		// LeBrasseur, hacked TAB -  http://www.snippets.org/CRC.TXT is probably a must-read.

		CRCLo = 0xFF;
		CRCHi = 0xFF;
		
		for (i = 0; i < (PACKETLENGTH-2) ; ++i)
		{
			tempChar1 = CRCLo ^ ReadPacket(i);
			CRCLo = CRCHi ^ CRCTableLo[tempChar1];
			CRCHi = CRCTableHi[tempChar1];
		}
		
		Write2Packet(PKT_CRC0, CRCLo);
		Write2Packet(PKT_CRC1, CRCHi);

		//------------------------------------------------------------------------------------
		// Flag that we can start transmitting THIS packet.

			if (gbStartTx)				// Packets should be done transmitting - fault if they aren't
				Fault(1);

			gTxPacketNum = gPacketNum;		// Start transmitting the packet we just stuffed
			gbStartTx = 1;					
						
		//------------------------------------------------------------------------------------
		// switch to the next packet and flag that we're ready to do this all over again.

		//---------------------------------
		#ifdef	GPACKETSWITCHONRD6
			RD6 = 1;
		#endif
		//---------------------------------

		if (gPacketNum >= (NUMPACKETS-1))
			gPacketNum = 0;
		else
			++gPacketNum;

		gbStartMain = 0;

		//---------------------------------
		#ifdef	GPACKETSWITCHONRD6
			RD6 = 0;
		#endif
		//---------------------------------

	}
}



