/*-------------------------------------------------------------------------------------------------\
| Solarboy Solar Bike Controller
|
| File:					BIKE.H			Header File
|
| Global Routines:	None
|
| Local Routines:		None
\-------------------------------------------------------------------------------------------------*/


//-------------------------------------------------------------------------------------------------
//  Global #defines
//-------------------------------------------------------------------------------------------------

#define	TRUE						 1
#define	FALSE						 0

#define	PACKETLENGTH			45
#define	NUMPACKETS    			 4
#define	PACKETS_TX_PER_SEC	 4

#define	FAULTLOGLENGTH			16

// DEBUG #DEFINES

//#define	FREEZEONFAULT
//#define	GOTGPSTOD7
//#define	ISRTIMING2PORTD
//#define	GPACKETSWITCHONRD6
//#define	FAULTTOPORTD

//-------------------------------------------------------------------------------------------------
//  Global Function Declarations
//-------------------------------------------------------------------------------------------------

	void	interrupt	ISR				(void);							// "interrupt" is a PICC keyword for the ISR
	void					Initialize		(void);							// Initialize the damn thing

	void					Fault				(char FaultNumberIn);		// Logging Fault routine
	void					IsrFault			(char FaultNumberIn);		// Logging Fault routine

	void 					Write2Packet		(char index, char data);	// Used to write to the current packet
	void 					IsrWrite2Packet	(char index, char data);	// Used to write to the current packet (ISR routines only)
	char					ReadPacket			(char index);					// Used to read from the current packet 
	char					IsrReadPacket		(char index);					// Used to read from the current packet (ISR routines only)

//-------------------------------------------------------------------------------------------------
//  Global Variables Declarations
//-------------------------------------------------------------------------------------------------

	extern	bit			gbPowerUp;				// Flag that we're powering up.

	extern	bit			gbGPSOk;					// 1 = GPS operating / 0 = not operating (timer2 watchdogs by setting to zero)
	extern	bit			gbStartTx;				// 1 = start packet transmissions / 0 = packet transmissions done
	extern	bit			gbStartMain;			// 1 = start main's processing of packet / 0 = packet processing done

	extern	bit			gbSystemFault;			// Flag that something went horribly, horribly wrong (& displays #s in digital byte of packet)

	extern	bit			gbPosBattAmps;			// 1 = positive battery current (charging) / 0 = negative (discharging)
	extern	bit			gbPosMotorAmps;		// 1 = positive motor current (running the motor) / 0 = negative (regen)

	extern	bit			gbSpeedSensor;			// 1 = Speed sensor has gone 0 to 1 / 0 = not yet
	extern	bit			gbCadenceSensor;		// 1 = cadence sensor has gone 0 to 1 / 0 = not yet

	extern	bank2	char	gPacket0[PACKETLENGTH];		// Arrays of bytes to send out the UART
	extern	bank2	char	gPacket1[PACKETLENGTH];		//
	extern	bank3	char	gPacket2[PACKETLENGTH];		//
	extern	bank3	char	gPacket3[PACKETLENGTH];		//

	extern unsigned char		gPacketNum;					// Which packet we're currently working on (loading)
	extern unsigned char		gTxPacketNum;				// Which packet we're transmitting

	extern signed long	ampHours;						// amp hours in "amp ticks"

	extern char				battAmps;						// High byte of battery amps count
	extern char				motorAmps;						// High byte of motor amps count

	extern unsigned char		speed0;						// Low byte of speed time between pulses
	extern unsigned char		cadence0;					// Low byte of cadence time between pulses
	extern unsigned char		speedCadence1;				// High nibbles of speed and cadence packed together

	extern unsigned char		gFaultLog[16];				// An array to store the fault log in.
	
//-------------------------------------------------------------------------------------------------
//  Define Macros
//-------------------------------------------------------------------------------------------------

// None

//-------------------------------------------------------------------------------------------------
//  Other Includes
//-------------------------------------------------------------------------------------------------

#include "bikepkt.h"


