/*-------------------------------------------------------------------------------------------------\
| Solarboy Solar Bike Controller
|
| File:					BIKE_LCD.C		Optrex LCD Routines
|
| Global Routines:	--
|
| Local Routines:		---
|
| Notes: Modified code from Hi-Tech C example files - rewritten for a 20MHz PIC16F877 using PORTD.
|			Should work on most Hitachi HD44780 controllers in 4bit mode.
|
| Hardware Setup: TO BE CHANGED TO 4BIT MODE!!!!!
|
|			RD0 - RD7: DB0 - DB7
|
|			RE0: RS
|			RE1: R/W*
|			RE2: E
|
\-------------------------------------------------------------------------------------------------*/

#include		<pic.h>
#include		"bike.h"

//-------------------------------------------------------------------------------------------------
//  Define Locally Global Variables (variables accessible by functions in this file only)
//-------------------------------------------------------------------------------------------------

	static	bit	LCD_RS	@ ((unsigned)&PORTE*8+0);	// Data/Register Select*
	static	bit	LCD_RW	@ ((unsigned)&PORTE*8+1);	// Read/Write*
	static	bit	LCD_EN	@ ((unsigned)&PORTE*8+2);	// Enable

//-------------------------------------------------------------------------------------------------
//  Local #defines
//-------------------------------------------------------------------------------------------------

	// NONE

//-------------------------------------------------------------------------------------------------	
//  Local Function Prototypes
//-------------------------------------------------------------------------------------------------

	void	lcd_write 	(unsigned char, unsigned char);



// ---------------------------------------------------------------
// Write a byte to the 4bit LCD interface
// --------------------------------------------------------------

void	lcd_write	(unsigned char c, char flgWriteType )
{
	int	temp;

	// First, check to see if we can write to the module yet (pause until ok)

	do {
	
		LCD_RW = 1;						// Read Command

		LCD_RS = 0;						// Read a register (not a character)

		TRISD = 0xFF;					// PORTD is all inputs

		LCD_EN = 1;						// Enable the output, read the bus
		asm("nop");
		temp = PORTD;
		LCD_EN = 0;

		TRISD = 0x00;					// Switch PORTD back to outputs
		
	} while (temp & (1<<7) );
	
	// Not busy. Write to the module.
	
	LCD_RW = 0;						// Write Command

	if ( flgWriteType > 0 )
		LCD_RS = 1;					// Write to data (1) (characters to display)	
	else
		LCD_RS = 0;					// Write to registers (0) (control registers)
	
	PORTD = c;						// Put data on bus
	LCD_EN = 1;						// Strobe the data into the LCD
	asm("nop");
	LCD_EN = 0;	
}

// ---------------------------------------------------------------
// Clear and Home the Display
// --------------------------------------------------------------

void	lcd_clear	(void)
{	
	lcd_write(0x01, 0);
}

/* write a string of chars to the LCD */


// ---------------------------------------------------------------
// Write a string to the display
// --------------------------------------------------------------

void	lcd_puts	(const char * s)
{
	while(*s)
		lcd_write(*s++,1);
}

// ---------------------------------------------------------------
// Write a character to the display
// --------------------------------------------------------------

void	lcd_putch	(char c)
{
	lcd_write(c,1);
}


// ---------------------------------------------------------------
// Goto a cursor position
// --------------------------------------------------------------

void	lcd_goto	(unsigned char pos)
{
	lcd_write(0x80+pos, 0);
}



// ---------------------------------------------------------------
// Print n spaces
// --------------------------------------------------------------

void	lcd_spaces	(unsigned char n)
{
	while (n)
	{
		--n;
		lcd_write(' ', 1);
	}
}

// ---------------------------------------------------------------
// Initialize the damn thing
// --------------------------------------------------------------

void	lcd_init	(void)
{

	lcd_write(0b00111000, 0);		// Function Set
			 	// ||||||||
			 	// |||||||+----------- Not used
		    	// ||||||+------------ Not used
		    	// |||||+------------- Character size	(0 = 5x8, 1 = 5x10)
		    	// ||||+-------------- # display lines	(0 = 1/8 duty for 1 line, 1 = 1/16 duty for 2 lines)
		    	// |||+--------------- #bits 				(0 = 4bit interface, 1 = 8bit interface)
			 	// ||+---------------- 1
		    	// |+----------------- 0
	       	// +------------------ 0

	lcd_write(0b00010000, 0);		// Cursor/display shift
			 	// ||||||||
			 	// |||||||+----------- Not used
		    	// ||||||+------------ Not used
		    	// |||||+------------- Shift R/L			(0 = Left, 1 = Right)
		    	// ||||+-------------- Shift C/D			(0 = move cursor, 1 = shift display)
		    	// |||+--------------- 1
			 	// ||+---------------- 0
		    	// |+----------------- 0
	       	// +------------------ 0

	lcd_write(0b00000110, 0);		// Entry mode set
			 	// ||||||||
			 	// |||||||+----------- Shift				(0 = no shift, 1 = shift)
		    	// ||||||+------------ Cursor Pos.		(0 = decrement, 1 = increment)
		    	// |||||+------------- 1
		    	// ||||+-------------- 0
		    	// |||+--------------- 0
			 	// ||+---------------- 0
		    	// |+----------------- 0
	       	// +------------------ 0

	lcd_write(0b00001100, 0);		// Display On/Off control
			 	// ||||||||
			 	// |||||||+----------- Cursor blink 	(0 = off, 1 = on)
		    	// ||||||+------------ Cursor				(0 = off, 1 = on)
		    	// |||||+------------- Display			(0 = off, 1 = on)
		    	// ||||+-------------- 1
		    	// |||+--------------- 0
			 	// ||+---------------- 0
		    	// |+----------------- 0
	       	// +------------------ 0

}




/*-------------------------------------------------------------------------------------------------\
| Function:		lcd_text
|
| Purpose:		Fill in all the non-changing text in the back of the LCD
|
| Discussion:	Prep the LCD so the power meter code doesn't need to deal with nonchanging stuff.
|					-
|
| Expects:		-
|					-
|
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 09/26/01 | Created.
\--------------------------------------------------------------------------------------------------*/


void lcd_text (void)
{
	//char *pTopLine =    "V=+000.00V Power=+00.00Kw";
	//char *pBottomLine = "I=+000.00V Cost/Hr=$+0.00";

	lcd_clear();
	
	lcd_puts("   0.0V    0.0A   0.00Kw");		// Print the top line.
	lcd_goto(0x40);									// Goto the the bottom line.
	lcd_puts("   Cost/Hour = $ 0.00   ");		// Print the bottom line


// OLD CODE

	//	lcd_puts("V= 000.0V Power=+00.00Kw");		// Print the top line.
	//	lcd_goto(0x40);									// Goto the the bottom line.
	//	lcd_puts("I=+000.0A Cost/Hr=$+0.00");		// Print the bottom line.
	//lcd_putch('h');
	//lcd_putch('e');
	//lcd_putch('l');
	//lcd_putch('l');
	//lcd_putch('o');
	//lcd_putch('?');

}


