/*-------------------------------------------------------------------------------------------------\
| Solarboy Solar Bike Controller
|
| File:					BIKE_MSC.C		Misc. Functions
|
| Global Routines:	SEE ALL BELOW
| 
| Local Routines:		NONE
|							
\-------------------------------------------------------------------------------------------------*/

#include	<pic.h>
#include "bike.h"

//-------------------------------------------------------------------------------------------------
//  Define Locally Global Variables (variables accessible by functions in this file only)
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local #defines
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local Function Prototypes
//-------------------------------------------------------------------------------------------------

	// None






/*-------------------------------------------------------------------------------------------------\
| Function:		Write2Packet
|
| Purpose:		Write data to a packet using gPacketNum
|
| Discussion:	Since we can't use pointers worth a damn, we have to use a function to write to
|					packets.
|
| Expects:	gPacketNum to point to the current packet.
|				index to have a reasonable value.
|
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 09/29/01 | Created.
\--------------------------------------------------------------------------------------------------*/

void Write2Packet (char index, char data)
{
	if (gPacketNum == 0)
		gPacket0[index] = data;
	
	else if (gPacketNum == 1)
		gPacket1[index] = data;

	else if (gPacketNum == 2)
		gPacket2[index] = data;

	else if (gPacketNum == 3)
		gPacket3[index] = data;

}


/*-------------------------------------------------------------------------------------------------\
| Function:		IsrWrite2Packet
|
| Purpose:		Write data to a packet using gPacketNum - ONLY for ISR functions
|
| Discussion:	Since we can't use pointers worth a damn, we have to use a function to write to
|					packets. Also, since the main loop is writing to packets, we have to have a separate
|					routine for the ISR. In particular, the GPS bytes. MAKE SURE THAT THE ISR VERSION AND
|					THE NON ISR VERSION NEVER WRITE TO THE SAME BYTE!
|
| Expects:	gPacketNum to point to the current packet.
|				index to have a reasonable value.
|
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 10/30/01 | Created.
\--------------------------------------------------------------------------------------------------*/

void IsrWrite2Packet (char index, char data)
{
	if (gPacketNum == 0)
		gPacket0[index] = data;
	
	else if (gPacketNum == 1)
		gPacket1[index] = data;

	else if (gPacketNum == 2)
		gPacket2[index] = data;

	else if (gPacketNum == 3)
		gPacket3[index] = data;

}


/*-------------------------------------------------------------------------------------------------\
| Function:		ReadPacket
|
| Purpose:		Read data from a packet using gPacketNum
|
| Discussion:	Since we can't use pointers worth a damn, we have to use a function to read from
|					packets.
|
| Expects:	gPacketNum to point to the current packet.

|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 09/29/01 | Created.
\--------------------------------------------------------------------------------------------------*/

char ReadPacket (char index)
{
	if (gPacketNum == 0)
		return gPacket0[index];
	
	else if (gPacketNum == 1)
		return gPacket1[index];

	else if (gPacketNum == 2)
		return gPacket2[index];

	else if (gPacketNum == 3)
		return gPacket3[index];
	else
		return 0;
}



/*-------------------------------------------------------------------------------------------------\
| Function:		IsrReadPacket
|
| Purpose:		Read data from a packet using gPacketNum
|
| Discussion:	Since we can't use pointers worth a damn, we have to use a function to read from
|					packets. TO BE CALLED ONLY BY ISR FUNCTIONS.
|
| Expects:	gPacketNum to point to the current packet.

|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 10/30/01 | Created.
\--------------------------------------------------------------------------------------------------*/

char IsrReadPacket (char index)
{
	if (gPacketNum == 0)
		return gPacket0[index];
	
	else if (gPacketNum == 1)
		return gPacket1[index];

	else if (gPacketNum == 2)
		return gPacket2[index];

	else if (gPacketNum == 3)
		return gPacket3[index];
	else
		return 0;
}



/*-------------------------------------------------------------------------------------------------\
| Function:		Fault
|
| Purpose:		Log and flag that a fault has occurred (ONLY CALLED FROM MAINLINE FUNCTIONS)
|
| Discussion:	Turns on RD7 (red LED) and flags a global flag. 
|					This routine is mostly for debugging.
|					1 = gbStartTx overflow in Main
|					2 = 
|					3 =
|					4 =
|					5 =
|
| Expects:		A char with a distinct error number.
|
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 11/07/01 | Created.
\--------------------------------------------------------------------------------------------------*/

void Fault (char FaultNumberIn)
{
	static unsigned char i;

	// Turn off interrupts so we can access the global fault log which is also
	// accessed in IsrFault.
	
	GIE = 0;

	// Log the error number to the first available spot in the  global fault log
	// This logged number will eventually be swapped with the "digital port byte"
	// in the event of a fault.
	
	i = 0;
	while ((gFaultLog[i] != 0) && (i < FAULTLOGLENGTH))
		++i;

	// We found an index which is within bounds and is zero (not yet used) so
	// write the error number there.
		
	if (i < FAULTLOGLENGTH)
		gFaultLog[i] = FaultNumberIn;

	gbSystemFault = 1;		// Global flag that the world sucks
	RD7 = 1;						// Shine the red LED of suckiness
	
	//---------------------------------
	#ifdef	FAULTTOPORTD
	if (RD6)
		FaultNumberIn &= 0b10111111;
	else
		FaultNumberIn |= 0b01000000;
	PORTD = FaultNumberIn;
	#endif
	//---------------------------------		

	//---------------------------------
	#ifdef	FREEZEONFAULT
	while (TRUE)
		;
	#endif
	//---------------------------------		

	GIE = 1;
}


/*-------------------------------------------------------------------------------------------------\
| Function:		IsrFault
|
| Purpose:		Log and flag that a fault has occurred (ONLY CALLED FROM ISR FUNCTIONS)
|
| Discussion:	Turns on RA2 (for an LED) and logs which error number it was.
|					This routine is mostly for debugging.
|					1 =
|					2 = gbStartMain overflow from Receive (GPS)
|					3 = gbStartMain overflow from Timer2 (backup 1s clock)
|					4 =
|					5 =
|
| Expects:		A char with a distinct error number.
|
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 09/29/01 | Created.
\--------------------------------------------------------------------------------------------------*/

void IsrFault (char FaultNumberIn)
{
	static	char	i;

	// Log the error number to the first available spot in the  global fault log
	// This logged number will eventually be swapped with the "digital port byte"
	// in the event of a fault.
	
	i = 0;
	while ((gFaultLog[i] != 0) && (i < FAULTLOGLENGTH))
		++i;

	// We found an index which is within bounds and is zero (not yet used) so
	// write the error number there.
		
	if (i < FAULTLOGLENGTH)
		gFaultLog[i] = FaultNumberIn;

	gbSystemFault = 1;		// Global flag that the world sucks
	RD7 = 1;						// Shine the red LED of suckiness

	//---------------------------------
	#ifdef	FAULTTOPORTD
	if (RD6)
		FaultNumberIn &= 0b10111111;
	else
		FaultNumberIn |= 0b01000000;
	PORTD = FaultNumberIn;
	#endif
	//---------------------------------		

	//---------------------------------
	#ifdef	FREEZEONFAULT
	while (TRUE)
		;
	#endif
	//---------------------------------		
}

